@@ -69,26 +69,38 @@ def dq_report(data, target=None, csv_engine="pandas", verbose=0):
         ext = os.path.splitext(data)[-1]
         # Load the file into a pandas dataframe based on the extension
         if ext == ".csv":
+            print("If large dataset, we will randomly sample 100K rows to speed up processing...")
             if csv_engine == 'pandas':
                 # Upload the data file into Pandas
-                df = pd.read_csv(data, nrows=1000000)
+                df = pd.read_csv(data)
             elif csv_engine == 'polars':
                 # Upload the data file into Polars
                 import polars as pl
-                df_polars = pl.read_csv(data, nrows=1000000)
+                df = pl.read_csv(data)
             elif csv_engine == 'parquet':
                 # Upload the data file into Parquet
                 import pyarrow as pa
-                df_parquet = pa.read_table(data)
+                df = pa.read_table(data)
+            else :
+                # print the pandas version
+                if str(pd.__version__)[0] == '2':
+                    print(f"pandas version={pd.__version__}. Hence using pyarrow backend.")
+                    df = pd.read_csv(data, engine='pyarrow', dtype_backend='pyarrow')
+                else:
+                    print(f"pandas version={pd.__version__}. Hence using pandas backend.")
+                    df = pd.read_csv(data)
         elif ext == ".parquet":
             df = pd.read_parquet(data)
-        elif ext in [".feather", ".arrow"]:
+        elif ext in [".feather", ".arrow", "ftr"]:
             df = pd.read_feather(data)
         else:
             print("Unsupported file format. Please use CSV, parquet, feather or arrow.")
-            return
+            return data
+        ######## This is to sample the data if it is too large ###
+        if df.shape[0] >= 1000000:
+            df = df.sample(100000)
     elif isinstance(data, pd.DataFrame):
-        # Use the input dataframe as is
+        print("We use the input dataframe as is...")
         df = data
     else:
         print("Invalid input. Please provide a string (filename) or a dataframe.")
@@ -112,22 +124,29 @@ def dq_report(data, target=None, csv_engine="pandas", verbose=0):
     )
 
     missing_values = df.isnull().sum()
+    missing_values_pct = ((df.isnull().sum()/df.shape[0])*100)
     missing_cols = missing_values[missing_values > 0].index.tolist()
     number_cols = df.select_dtypes(include=["integer", "float"]).columns.tolist() # Get numerical columns
     float_cols = df.select_dtypes(include=[ "float"]).columns.tolist() # Get float columns
+    id_cols = []
+    zero_var_cols = []
 
     missing_data = pd.DataFrame(
-        missing_values,
-        columns=['Missing Values']
+        missing_values_pct,
+        columns=['Missing Values%']
     )
     unique_values = pd.DataFrame(
-        columns=['Unique Values']
+        columns=['Unique Values%']
     )
     for row in list(df.columns.values):
         if row in float_cols:
             unique_values.loc[row] = ["NA"]
         else:
-            unique_values.loc[row] = [df[row].nunique()]
+            unique_values.loc[row] = [int(100*df[row].nunique()/df.shape[0])]
+            if df[row].nunique() == df.shape[0]:
+                id_cols.append(row)
+            elif df[row].nunique() == 1:
+                zero_var_cols.append(row)
         
     maximum_values = pd.DataFrame(
         columns=['Maximum Value']
@@ -153,6 +172,36 @@ def dq_report(data, target=None, csv_engine="pandas", verbose=0):
     dq_df2["first_comma"] = ""
     dq_df2[new_col] = f""
     
+    # Detect ID columns in dataset and recommend removing them
+    if len(id_cols) > 0:
+        new_string = f"There are ID columns in the dataset. Recommend removing them before modeling."
+        dq_df1.loc[bad_col,new_col] += dq_df1.loc[bad_col,'first_comma'] + new_string
+        dq_df1.loc[bad_col,'first_comma'] = ', '
+        for col in id_cols:
+            # Append a row to the dq_df1 with the column name and the issue only if the column has a missing value
+            new_string = f"Possible ID colum: drop before modeling process."
+            dq_df2.loc[col,new_col] += dq_df2.loc[col,'first_comma'] + new_string
+            dq_df2.loc[col,'first_comma'] = ', '
+    else:
+        new_string = f"There are no ID columns in the dataset. So no ID columns to remove before modeling."
+        dq_df1.loc[good_col,new_col] += dq_df1.loc[good_col,'first_comma'] + new_string
+        dq_df1.loc[good_col,'first_comma'] = ', '
+
+    # Detect ID columns in dataset and recommend removing them
+    if len(zero_var_cols) > 0:
+        new_string = f"There are zero-variance columns in the dataset. Recommend removing them before modeling."
+        dq_df1.loc[bad_col,new_col] += dq_df1.loc[bad_col,'first_comma'] + new_string
+        dq_df1.loc[bad_col,'first_comma'] = ', '
+        for col in zero_var_cols:
+            # Append a row to the dq_df1 with the column name and the issue only if the column has a missing value
+            new_string = f"Zero-variance colum: drop before modeling process."
+            dq_df2.loc[col,new_col] += dq_df2.loc[col,'first_comma'] + new_string
+            dq_df2.loc[col,'first_comma'] = ', '
+    else:
+        new_string = f"There are no zero-variance columns in the dataset. So no zero-variance columns to remove before modeling."
+        dq_df1.loc[good_col,new_col] += dq_df1.loc[good_col,'first_comma'] + new_string
+        dq_df1.loc[good_col,'first_comma'] = ', '
+
     # Detect missing values and suggests to impute them with mean, median, mode, or a constant value123
     #missing_values = df.isnull().sum()
     #missing_cols = missing_values[missing_values > 0].index.tolist()
@@ -274,7 +323,7 @@ def dq_report(data, target=None, csv_engine="pandas", verbose=0):
                     dq_df1.loc[bad_col,'first_comma'] = ', '
                     first_time =False
                 ### check if there are outlier columns and print them ##
-                new_string = f"has {len(outliers)} outliers greater than upper bound ({upper_bound}) or lower than lower bound: {lower_bound}. Cap them or remove them."
+                new_string = f"has {len(outliers)} outliers greater than upper bound ({upper_bound}) or lower than lower bound({lower_bound}). Cap them or remove them."
                 dq_df2.loc[col,new_col] += dq_df2.loc[col,'first_comma'] + new_string
                 dq_df2.loc[col,'first_comma'] = ', '
         if len(outlier_cols) < 1:
@@ -291,7 +340,7 @@ def dq_report(data, target=None, csv_engine="pandas", verbose=0):
         dq_df1.loc[bad_col,new_col] += dq_df1.loc[bad_col,'first_comma'] + new_string
         dq_df1.loc[bad_col,'first_comma'] = ', '
         for col in high_card_cols:
-            new_string = f"high cardinality with {cardinality[col]} unique values."
+            new_string = f"high cardinality with {cardinality[col]} unique values: Use hash encoding or embedding to reduce dimension."
             dq_df2.loc[col,new_col] += dq_df2.loc[col,'first_comma'] + new_string
             dq_df2.loc[col,'first_comma'] = ', '
     else:
@@ -566,13 +615,13 @@ class Fix_DQ(BaseEstimator, TransformerMixin):
         # Drop duplicate rows
         dup_rows = X.duplicated().sum()
         if dup_rows > 0:
-            print(f'Alert: Dropping {dup_rows} rows can sometimes cause column data types to change to object. Double-check!')
+            print(f'Alert: Dropping {dup_rows} duplicate rows can sometimes cause column data types to change to object. Double-check!')
             X = X.drop_duplicates()
         
         # Drop duplicate columns
         dup_cols = X.T.duplicated().sum()
         if dup_cols > 0:
-            print(f'Alert: Dropping {dup_cols} cols can sometimes cause column data types to change to object. Double-check!')
+            print(f'Alert: Dropping {dup_cols} duplicate cols can sometimes cause column data types to change to object. Double-check!')
             X = X.T.drop_duplicates().T
         
         # Return the DataFrame with no duplicates
@@ -587,16 +636,12 @@ class Fix_DQ(BaseEstimator, TransformerMixin):
         
         # Get the numerical columns
         num_cols = X.select_dtypes(include=["float"]).columns.tolist()
-        
-        # Define a threshold for skewness
-        skew_threshold = 0.5
-        
+                
         # Loop through each numerical column
         for col in num_cols:
             # Find if a column transformer exists for this column
             if col in self.col_transformers_:
                 # Cap the outliers using the upper bound
-                # Check if the skewness is above the threshold
                 if str(self.col_transformers_[col]).split("(")[0] == "PowerTransformer":
                     ### power transformer expects Pandas DataFrame
                     pt = self.col_transformers_[col]
@@ -619,6 +664,17 @@ class Fix_DQ(BaseEstimator, TransformerMixin):
         # Get the numerical columns
         num_cols = X.select_dtypes(include=["int", "float"]).columns.tolist()
         float_cols = X.select_dtypes(include=["float"]).columns.tolist()
+        non_float_cols = left_subtract(X.columns, float_cols)
+
+        # Detect ID columns
+        self.id_cols_ = [column for column in non_float_cols if X[column].nunique() == X.shape[0]]
+        if len(self.id_cols_) > 0:
+            print(f"{len(self.id_cols_)} ID cols will be dropped from further processing: {self.id_cols_}")
+
+        # Detect zero-variance columns
+        self.zero_var_cols_ = [column for column in non_float_cols if X[column].nunique() == 1]
+        if len(self.zero_var_cols_) > 0:
+            print(f"    {len(self.zero_var_cols_)} zero-variance cols will be dropped from further processing: {self.zero_var_cols_}")
         
         # Detect highly correlated features
         self.drop_corr_cols_ = []
@@ -664,7 +720,7 @@ class Fix_DQ(BaseEstimator, TransformerMixin):
         self.col_transformers_ = {}
         
         # Define a threshold for skewness
-        skew_threshold = 0.5
+        skew_threshold = 1.0
         
         # Loop through each float column
         for col in float_cols:
@@ -709,9 +765,23 @@ class Fix_DQ(BaseEstimator, TransformerMixin):
             # Convert X to a pandas DataFrame
             X = pd.DataFrame(X)
         
+        ### drop mixed data type columns from further processing ##
+        if len(self.id_cols_) > 0:
+            X = X.drop(self.id_cols_, axis=1)
+
+        ### drop mixed data type columns from further processing ##
+        if len(self.zero_var_cols_) > 0:
+            X = X.drop(self.zero_var_cols_, axis=1)
+
         ### drop mixed data type columns from further processing ##
         if len(self.mixed_type_cols_) > 0:
-            X = X.drop(self.mixed_type_cols_, axis=1)
+            drop_cols = left_subtract(self.mixed_type_cols_, self.zero_var_cols_+self.id_cols_)
+            if len(drop_cols) > 0:
+                X = X.drop(drop_cols, axis=1)
+            else:
+                drop_cols = left_subtract(self.zero_var_cols_+self.id_cols_, self.mixed_type_cols_)
+                if len(drop_cols) > 0:
+                    X = X.drop(drop_cols, axis=1)
             
         ### drop highly correlated columns from further processing ##
         if len(self.drop_corr_cols_) > 0:
@@ -749,7 +819,7 @@ class Fix_DQ(BaseEstimator, TransformerMixin):
 
 ############################################################################################
 module_type = 'Running' if  __name__ == "__main__" else 'Imported'
-version_number =  '1.6'
+version_number =  '1.8'
 print(f"""{module_type} pandas_dq ({version_number}). Always upgrade to get latest version.
 from pandas_dq import dq_report, Fix_DQ
 """)
